﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace Color_Slicing_Project
{
    public partial class ColorSlicing : Form
    {
        public ColorSlicing()
        {
            InitializeComponent();
            ofd = new OpenFileDialog();
            sfd = new SaveFileDialog();
            ofd.Filter = filter;
            sfd.Filter = filter;
        }

        Bitmap image, processed, display;
        string image_path;
        string filter = "Image | *.png; *.jpg; *.gif";
        OpenFileDialog ofd;
        SaveFileDialog sfd;
        byte[] image_bytes;
        int stride;
        byte[] prototype_color;


        private void image_frame_Click(object sender, EventArgs e)
        {
            MouseEventArgs me = (MouseEventArgs)e;
            Point coordinates = me.Location;
            prototype_color = new byte[3];
            for (int i = 0; i < 3; i++)
            {
                int loc = coordinates.X * 3 + coordinates.Y * stride;
                prototype_color[i] = image_bytes[loc + i];
            }
            processed = processed.ColorSlice(prototype_color, 20);
            display = processed.ScaleImage(result_frame.Width, result_frame.Height);
            result_frame.Image = display;
            processed = image;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sfd.FileName = image_path;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                processed.Save(sfd.FileName);
            }
        }

        private void loadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                image_path = ofd.FileName;
                image = (Bitmap)Bitmap.FromFile(image_path);
                processed = image;
                display = image.ScaleImage(image_frame.Width, image_frame.Height);
                image_frame.Image = display;

                int w = processed.Width;
                int h = processed.Height;

                BitmapData image_data = processed.LockBits(
                    new Rectangle(0, 0, w, h),
                    ImageLockMode.ReadOnly,
                    PixelFormat.Format24bppRgb);

                stride = image_data.Stride;
                int bytes = image_data.Stride * image_data.Height;
                image_bytes = new byte[bytes];
                Marshal.Copy(image_data.Scan0, image_bytes, 0, bytes);
                processed.UnlockBits(image_data);
            }
        }

        
    }

    public static class Ext
    {
        public static Bitmap ScaleImage(this Bitmap image, int width, int height)
        {
            float xratio = (float)width / image.Width;
            float yratio = (float)height / image.Height;
            float ratio = Math.Min(xratio, yratio);

            int w = (int)(image.Width * ratio);
            int h = (int)(image.Height * ratio);

            Bitmap result = new Bitmap(image, new Size(w, h));

            return result;
        }

        public static Bitmap ColorSlice(this Bitmap image, byte[] prototype_color, float color_cube_width)
        {
            int w = image.Width;
            int h = image.Height;

            BitmapData image_data = image.LockBits(
                new Rectangle(0, 0, w, h),
                ImageLockMode.ReadOnly,
                PixelFormat.Format24bppRgb);

            int bytes = image_data.Stride * image_data.Height;
            byte[] buffer = new byte[bytes];
            byte[] result = new byte[bytes];
            Marshal.Copy(image_data.Scan0, buffer, 0, bytes);
            image.UnlockBits(image_data);

            //subtracting 3 from total amount of bytes will let you process images with odd number dimensions
            for (int i = 0; i < bytes - 3; i+=3)
            {
                for (int c = 0; c < 3; c++)
                {
                    if (Math.Abs(buffer[i + c] - prototype_color[c]) > (color_cube_width / 2))
                    {
                        result[i + c] = 128;
                    }
                    else
                    {
                        result[i + c] = buffer[i + c];
                    }
                }
            }

            Bitmap res_image = new Bitmap(w, h);
            BitmapData result_data = res_image.LockBits(
                new Rectangle(0, 0, w, h),
                ImageLockMode.WriteOnly,
                PixelFormat.Format24bppRgb);
            Marshal.Copy(result, 0, result_data.Scan0, bytes);
            res_image.UnlockBits(result_data);
            return res_image;
        }
    }
}
